// SetWindow.cpp

#include <Application.h>
#include <CheckBox.h>
#include <Box.h>
#include <Button.h>
#include <RadioButton.h>
#include <ColorControl.h>
#include <MenuField.h>
#include <Menu.h>
#include <MenuItem.h>

#include <FilePanel.h>
#include <Path.h>
#include <Entry.h>
#include <TextControl.h>
#include <Screen.h>

#include <stdio.h>

#include "states.h"
#include "cd.h"
#include "SetWindow.h"
#include "MsgVals.h"


//Global settings
extern D_Cd					*cd;
extern D_STATES				states;
extern BWindow				*handler;
extern SetWindow			*setWindow;			//window for settings
extern char 				AppPath[B_PATH_NAME_LENGTH+B_FILE_NAME_LENGTH];
extern BBitmap				*ButtonsBitmap;				//bitmap of buttons
extern	int LoadBitmap(const char *file,BBitmap **pic);

SetWindow::SetWindow()
	: BWindow(BRect(0,0,380,240), "Settings",B_FLOATING_WINDOW_LOOK,B_NORMAL_WINDOW_FEEL,
			  B_NOT_RESIZABLE | B_NOT_ZOOMABLE | B_WILL_ACCEPT_FIRST_CLICK)
{


//Position Settings window so it is always on the screen
	BRect		MainAppFrame,MainAppBounds,SettingsWinFrame,screen;
    BPoint 		newPosition;
    MainAppFrame =	handler->Frame();
	MainAppBounds = handler->Bounds();	//window cords for main app
	SettingsWinFrame = Frame();	
	
	BScreen *theScreen = new BScreen(B_MAIN_SCREEN_ID);
	screen = theScreen->Frame();

	newPosition.x = MainAppFrame.left;
	newPosition.y = MainAppFrame.bottom + 24;
	
	if( MainAppFrame.left < 0 )
		newPosition.x = 1;

	if( MainAppFrame.right > screen.right)
		newPosition.x = screen.right - (SettingsWinFrame.right - SettingsWinFrame.left);
		
	if( (MainAppFrame.left + (SettingsWinFrame.right - SettingsWinFrame.left )) > screen.right)
		newPosition.x = screen.right - (SettingsWinFrame.left + SettingsWinFrame.right + 2);

	if( (SettingsWinFrame.bottom + MainAppFrame.bottom +MainAppBounds.bottom) > screen.bottom ) 
		newPosition.y = MainAppFrame.top - (SettingsWinFrame.top + SettingsWinFrame.bottom +24);
		
	theScreen->~BScreen();		
	MoveTo(newPosition);		
// end Position Settings window so it is always on the screen	
	
	
	sv = new BView(Bounds(),"settingview", B_FOLLOW_ALL, B_WILL_DRAW);


	// Ring options setting box
	optionsBox = new BBox(BRect(10,10,370,230),"optionsbox");
	optionsBox->SetLabel("Options");
	sv->AddChild(optionsBox);
	
	// create checkbox for Shuffle Across CDs 
//	ShuffleAcross = new BCheckBox(BRect(10,15,140,30), "ShuffleCDs", "Shuffle across CDs",new BMessage(SHUFFLE_ACROSS_CHANGE));
//	ShuffleAcross->SetValue(states.ShuffleAcross);
//	optionsBox->AddChild(ShuffleAcross);

	// create checkbox for stop CD when changing disks 		   "Stop CD before changing discs"	
	//	        											   "Allow simultaneous playback on multiple devices"
//	StopChange = new BCheckBox(BRect(160,15,355,30), "StopCD", "Allow playing of multiple CDs at once",new BMessage(STOP_CHANGE));
//	StopChange->SetValue(states.DontStopCDWhenSwitching);
//	optionsBox->AddChild(StopChange);
	
	// create checkbox for reseting CD volume
	ResetVolume = new BCheckBox(BRect(10,15,160,30), "ResetVolume", "Reset CD volume on exit",new BMessage(RESET_VOLUME_CHANGE));
	ResetVolume->SetValue(states.ResetVolume);
	optionsBox->AddChild(ResetVolume);	
	
	// create checkbox for saving screen position of player
	SaveScreenPos = new BCheckBox(BRect(160,15,355,30), "SaveScreenPos", "Save screen position of player",new BMessage(SAVE_SCREEN_CHANGE));
	SaveScreenPos->SetValue(states.SaveScreenPosition);
	optionsBox->AddChild(SaveScreenPos);
	
	// create checkbox for shuffle by default			
	ShuffleDefault = new BCheckBox(BRect(10,30,160,45), "ShuffleDefault", "Shuffle mode by default",new BMessage(SHUFFLE_DEFAULT_CHANGE));
	ShuffleDefault->SetValue(states.ShuffleDefault);
	optionsBox->AddChild(ShuffleDefault);		
	
	// create checkbox for loop by default	
	LoopDefault = new BCheckBox(BRect(160,30,355,45), "LoopDefault", "Loop mode by default",new BMessage(LOOP_DEFAULT_CHANGE));
	LoopDefault->SetValue(states.LoopDefault);
	optionsBox->AddChild(LoopDefault);

	//Layout the Default CDROM drop down list
	BMenu 			*tempMenu;
	BMenuItem 		*tempMenuItem;	
	switch(states.DefaultDevice)
	{
		case 0:
		case 1:	tempMenu = new BMenu("CD Device1");
				break;
		case 2: tempMenu = new BMenu("CD Device2");
				break;
		case 3: tempMenu = new BMenu("CD Device3");
				break;
		default:
		case 4: tempMenu = new BMenu("CD Device4");
				break;
	}
	
	tempMenu->SetLabelFromMarked(true);
	if(states.NumDevices >=4)
	{
		tempMenuItem = new BMenuItem("CD Device1", new BMessage(DEVICE1_CHANGE));
		tempMenu->AddItem(tempMenuItem);
		tempMenuItem = new BMenuItem("CD Device2", new BMessage(DEVICE2_CHANGE));
		tempMenu->AddItem(tempMenuItem);
		tempMenuItem = new BMenuItem("CD Device3", new BMessage(DEVICE3_CHANGE));
		tempMenu->AddItem(tempMenuItem);
		tempMenuItem = new BMenuItem("CD Device4", new BMessage(DEVICE4_CHANGE));
		tempMenu->AddItem(tempMenuItem);						
	}
	if(states.NumDevices ==3)
	{
		tempMenuItem = new BMenuItem("CD Device1", new BMessage(DEVICE1_CHANGE));
		tempMenu->AddItem(tempMenuItem);
		tempMenuItem = new BMenuItem("CD Device2", new BMessage(DEVICE2_CHANGE));
		tempMenu->AddItem(tempMenuItem);
		tempMenuItem = new BMenuItem("CD Device3", new BMessage(DEVICE3_CHANGE));
		tempMenu->AddItem(tempMenuItem);	
	}
	if(states.NumDevices ==2)
	{
		tempMenuItem = new BMenuItem("CD Device1", new BMessage(DEVICE1_CHANGE));
		tempMenu->AddItem(tempMenuItem);
		tempMenuItem = new BMenuItem("CD Device2", new BMessage(DEVICE2_CHANGE));
		tempMenu->AddItem(tempMenuItem);	
	}
	if(states.NumDevices ==1)
	{
		tempMenuItem = new BMenuItem("CD Device1", new BMessage(DEVICE1_CHANGE));
		tempMenu->AddItem(tempMenuItem);	
	}
	
	BMenuField *mf = new BMenuField(BRect(10,55,185,70),"MenuField","Default CD device",tempMenu);
	mf->SetDivider(94);
	optionsBox->AddChild(mf);

	//Layout the Default time mode drop down list
	switch(states.DefaultTimeMode)
	{
		default:
		case 0: tempMenu = new BMenu("Total track played");
				break;
		case 1: tempMenu = new BMenu("Total disc played");
				break;
		case 2: tempMenu = new BMenu("Total track remaining");
				break;
		case 3: tempMenu = new BMenu("Total disc remaining");
				break;
	}
	
	tempMenu->SetLabelFromMarked(true);
	tempMenuItem = new BMenuItem("Total track played", new BMessage(TIME_MODE0_CHANGE));
	tempMenu->AddItem(tempMenuItem);
	tempMenuItem = new BMenuItem("Total disc played", new BMessage(TIME_MODE1_CHANGE));
	tempMenu->AddItem(tempMenuItem);
	tempMenuItem = new BMenuItem("Total track remaining", new BMessage(TIME_MODE2_CHANGE));
	tempMenu->AddItem(tempMenuItem);
	tempMenuItem = new BMenuItem("Total disc remaining", new BMessage(TIME_MODE3_CHANGE));
	tempMenu->AddItem(tempMenuItem);

	BMenuField *mf2 = new BMenuField(BRect(10,76,230,91),"MenuField","Default time mode",tempMenu);
	mf2->SetDivider(94);
	optionsBox->AddChild(mf2);

	//Create open file panel for later use
	file = new BFilePanel(B_OPEN_PANEL,NULL,NULL,B_DIRECTORY_NODE | B_SYMLINK_NODE,false,NULL,NULL,false,true);
	char *PanelPath = new char[strlen(AppPath)+strlen("skins/")+1];
	strcpy(PanelPath,AppPath);
	strcat(PanelPath,"skins/");
	file->SetPanelDirectory(PanelPath);
	delete PanelPath;	
	
	//Create textcontrol to show skin dir			
	skinPath = new BTextControl(BRect(10,104,330,119),"skinpath","Skin path","",NULL,B_FOLLOW_NONE);
	skinPath->SetText(states.SkinPath);		//set this to current path from config.file
	skinPath->TextView()->SetMaxBytes(B_PATH_NAME_LENGTH); //Make this the max value for DIR
	skinPath->SetDivider(45);
	skinPath->SetEnabled(false);
	optionsBox->AddChild(skinPath);	
	
	//Create select skin path button  	
	skinPathButton = new BButton(BRect(335,102,355,117), "skinpathbutton", "...", new BMessage(SKIN_PATH_CHANGE), B_FOLLOW_NONE);
	skinPathButton->SetFontSize(15.0);
	optionsBox->AddChild(skinPathButton);	


	//create radio buttons for color selection	
	VolumeColor = new BRadioButton(BRect(10,135,110,150),"Vol","Volume indicator color",new BMessage(COLOR_TARGET_CHANGE),B_FOLLOW_NONE);
	SelectedColor = new BRadioButton(BRect(119,135,224,150),"Sel","Selected CD color",new BMessage(COLOR_TARGET_CHANGE),B_FOLLOW_NONE);
	UnselectedColor = new BRadioButton(BRect(235,135,350,150),"Unsel","Unselected CD color",new BMessage(COLOR_TARGET_CHANGE),B_FOLLOW_NONE);	
	VolumeColor->SetValue(1);

	optionsBox->AddChild(VolumeColor);
	optionsBox->AddChild(SelectedColor);	
	optionsBox->AddChild(UnselectedColor);		
	
	//Create color control
	ColorControl = new BColorControl(BPoint(10,155),B_CELLS_32x8,6,"Color Selector",
			 						 new BMessage(COLOR_CHANGE),false);
	ColorControl->SetViewColor(216,216,216);
	ColorControl->SetValue(states.VolumeColor);
	optionsBox->AddChild(ColorControl);		

	//Create default color button  	
	ColorDefaultButton = new BButton(BRect(300,185,350,195), "Default", "Default", new BMessage(DEFAULT_COLOR_CHANGE), B_FOLLOW_NONE);
	optionsBox->AddChild(ColorDefaultButton);	
	
	// set the color to gray, add the child, and show the window 
	sv->SetViewColor(216,216,216);
	AddChild(sv);
	
	Show();	

	PostMessage(COLOR_TARGET_CHANGE);		//Draw color sample first time through

}

//////////////////////////////////////////////////////////////////////////////////

void SetWindow::WindowActivated(bool active)
{
	rgb_color	black;
	black.red	=0;
	black.green =0;
	black.blue	=0;
	PostMessage(COLOR_TARGET_CHANGE);
	FindView("optionsbox")->SetHighColor(black);
	optionsBox->SetLabel("Options");
}

//////////////////////////////////////////////////////////////////////////////////

void SetWindow::MessageReceived(BMessage *message)
{
	
	rgb_color	defaultColor,oldColor;
	entry_ref	ref;
	BPath		path;
	BEntry		entry;	
	int 		result=0;
	
	switch(message->what)
	{
		case COLOR_TARGET_CHANGE:
			if(VolumeColor->Value()==1)
			{
				ColorControl->SetValue(states.VolumeColor);
				FindView("optionsbox")->SetHighColor(states.VolumeColor);
			}
			if(SelectedColor->Value()==1)
			{
				ColorControl->SetValue(states.SelectedCDColor);
				FindView("optionsbox")->SetHighColor(states.SelectedCDColor);
			}
			if(UnselectedColor->Value()==1)
			{
				ColorControl->SetValue(states.UnselectedCDColor);
				FindView("optionsbox")->SetHighColor(states.UnselectedCDColor);	
	 		}
	 		oldColor = FindView("optionsbox")->HighColor();
	 		FindView("optionsbox")->FillRect(BRect(301,155,349,184));
	 		FindView("optionsbox")->SetHighColor(oldColor);
			break;
		
		case COLOR_CHANGE:
			if(VolumeColor->Value()==1)
	 		{
	 			states.VolumeColor = ColorControl->ValueAsColor(); 	
				FindView("optionsbox")->SetHighColor(states.VolumeColor);
			}
			if(SelectedColor->Value()==1)
	 		{
	 			states.SelectedCDColor = ColorControl->ValueAsColor(); 	
				FindView("optionsbox")->SetHighColor(states.SelectedCDColor);
			}
			if(UnselectedColor->Value()==1)
	 		{
	 			states.UnselectedCDColor = ColorControl->ValueAsColor(); 		 			 		
	 			FindView("optionsbox")->SetHighColor(states.UnselectedCDColor);
	 		}
	 		oldColor = FindView("optionsbox")->HighColor();
	 		FindView("optionsbox")->FillRect(BRect(301,155,349,184));
	 		FindView("optionsbox")->SetHighColor(oldColor);
	 		break;
	 	
	 	case DEFAULT_COLOR_CHANGE:
			if(VolumeColor->Value()==1)
	 		{
	 			defaultColor.red   = 0;
	 			defaultColor.green = 200;
	 			defaultColor.blue  = 0;
	 			states.VolumeColor = defaultColor;
			}
			if(SelectedColor->Value()==1)
			{
	 			defaultColor.red   = 255;
	 			defaultColor.green = 0;
	 			defaultColor.blue  = 0;			
	 			states.SelectedCDColor = defaultColor;
			}
			if(UnselectedColor->Value()==1)
			{
	 			defaultColor.red   = 0;
	 			defaultColor.green = 100;
	 			defaultColor.blue  = 0;			
	 			states.UnselectedCDColor = defaultColor;
	 		}			
	 		PostMessage(COLOR_TARGET_CHANGE);		//done so it will be updated in real time
	 		break;
	 	
	 //	case SHUFFLE_ACROSS_CHANGE:
	 	//	states.ShuffleAcross = ShuffleAcross->Value();
	 	//	break;
	 	
	// 	case STOP_CHANGE:
	 //		states.DontStopCDWhenSwitching = StopChange->Value();
	 	//	break;
	 	
	 	case SAVE_SCREEN_CHANGE:
	 		states.SaveScreenPosition = SaveScreenPos->Value();
	 		break;
	 	
	 	case RESET_VOLUME_CHANGE:
	 		states.ResetVolume = ResetVolume->Value();
	 		cd->ResetVolume(states.ResetVolume);
	 		break;
	 		
	 	case SHUFFLE_DEFAULT_CHANGE:
	 		states.ShuffleDefault = ShuffleDefault->Value();
	 		break;
	 		
		case LOOP_DEFAULT_CHANGE:
			states.LoopDefault = LoopDefault->Value();
			break;	 		
		
		case DEVICE1_CHANGE:
			states.DefaultDevice = 1;
			break;
			
		case DEVICE2_CHANGE:
			states.DefaultDevice = 2;
			break;
			
		case DEVICE3_CHANGE:
			states.DefaultDevice = 3;
			break;
			
		case DEVICE4_CHANGE:
			states.DefaultDevice = 4;
			break;									
		
		case TIME_MODE0_CHANGE:
			states.DefaultTimeMode = 0;
			break;
		
		case TIME_MODE1_CHANGE:
			states.DefaultTimeMode = 1;
			break;

		case TIME_MODE2_CHANGE:
			states.DefaultTimeMode = 2;
			break;					

		case TIME_MODE3_CHANGE:
			states.DefaultTimeMode = 3;
			break;		
		
		case SKIN_PATH_CHANGE:
			file->Show();
			break;

		case B_REFS_RECEIVED:		
			message->FindRef("refs",0,&ref);
			entry.SetTo(&ref,true);
			entry.GetPath(&path);
			strcpy(states.SkinPath,path.Path());
			strcat(states.SkinPath,"/");		//Need to add trailing '/' open panel does not add it ?
			
			result = LoadBitmap(states.SkinPath,&ButtonsBitmap);
			if(result == 1)			//Folder has a skin
				skinPath->SetText(states.SkinPath);
			else if(result == 2)	//No skin in folder use default
			{
				skinPath->SetText(states.SkinPath);		//SkinPath updated in LoadBitmap to default	
			}
			else	//Somehow no default skin.tga
			{
				be_app->PostMessage(B_QUIT_REQUESTED);
			}
			break;
		
	 	default:
	 		BWindow::MessageReceived(message);
	 		break;
	}
}

//////////////////////////////////////////////////////////////////////////////////

void SetWindow::Quit() 
{
	// alert the application of our impending demise 
	handler->PostMessage(SETWIN_QUIT);
	BWindow::Quit();
}

//////////////////////////////////////////////////////////////////////////////////

SetWindow::~SetWindow()
{
	file->~BFilePanel();
}
