#include "cd.h"

#include <Entry.h>

D_Cd::D_Cd(int initialDevice)
{

	DeviceNames = new BList(4);		//List of CDROM's

	media_node cdNode;
	CurrentTrack = 1;				//May not be needed but might as well
	ActiveDevice = 0;
	status_t result;
	SoundCardPresent = true;
	ResetVolumeLevel = true;
	CanChangeCDVolume =true;
	CurrentVolume[0] = 0;			//only using as if one channel
	
	FindCDDevices("/dev/disk");		//Make the list of available CD drives

	BMediaRoster* roster = BMediaRoster::Roster();
	result = roster->GetAudioOutput(&cdNode);
	
	if(result != B_OK)
		SoundCardPresent = false;
	
	result = roster->GetParameterWebFor(cdNode, &ParamWeb);
	
	if(result != B_OK)
		SoundCardPresent = false;	


	if(SoundCardPresent)
	{
		// figure out what "now" is in the performance time base associated
		// with the node that the BParameterWeb controls
		BTimeSource* timeSource = roster->MakeTimeSourceFor(ParamWeb->Node());

		bigtime_t now = timeSource->Now();

		// find the CD gain parameter
		int32 numParams = ParamWeb->CountParameters();
	
		BParameter* param = NULL;
		for (int32 i = 0; i < numParams; i++)
		{
			BParameter* p = ParamWeb->ParameterAt(i);
			if (!strcmp(p->Kind(),B_GAIN) && !strcmp("CD",p->Group()->Name()))
			{		
				param = p;
				break;
			}
		}

		// we need it to be the right type; this will print an error message
		// if we didn't find it

// -- TESTING FOR PPC VERSION
		if(param != NULL)	//there are CD volume controls to mess with
		{
			if (param->Type() == BParameter::B_CONTINUOUS_PARAMETER)
			{
				GainParam = dynamic_cast<BContinuousParameter*>(param);
			}
			else
			{
				CanChangeCDVolume =false;
				log("CD Error: Could not find CD volume in Media Settings.");
			}
		}
		else	//no CD volume controls to mess with
		{
			CanChangeCDVolume =false;
		}
		
	if(CanChangeCDVolume ==true)
	{

		//Save the users original CD volume settings
		float chanData[2];
		size_t	datasize;
		datasize = sizeof(chanData);

		//Get how much to increment volume by (dictated by sound card)
		StepValue =	GainParam->ValueStep();
	
		//hack for wierd values returned above
		if(StepValue < 0.5)
			StepValue = 0.5;
		else if(StepValue > 0.5 && StepValue < 1.0)
			StepValue = 1.0;
	//	if(StepValue < 1)
	//		StepValue = 1.0;

		// now set the BParameter!
		GainParam->GetValue(&chanData, &datasize, &now);

		// must delete the time source from BMediaRoster::MakeTimeSourceFor(...)
		timeSource->Release();		

		InitialVolume = chanData[0];	//save original CD volume

		CurrentVolume[0] = chanData[0];		//Set global for future use
	//	CurrentVolume[1] = chanData[1];		//Set global for future use	

		// calculate the minimum and maximum volume values for CD 
		MinVolume = GainParam->MinValue();

		MaxVolume = GainParam->MaxValue();
	

	}//end CanChangeCDVolume ==true
		
	}//end 	if SoundCardPresent 

	
	if(initialDevice ==1)
		Init(CDROM_1);			//Get the CD rom ready to rock
	if(initialDevice ==2)
		Init(CDROM_2);			//Get the CD rom ready to rock	
	if(initialDevice ==3)
		Init(CDROM_3);			//Get the CD rom ready to rock	
	if(initialDevice >=4)
		Init(CDROM_4);			//Get the CD rom ready to rock	
		
		
}

bool D_Cd::Init(int whichCD)
{


	//Stop();				//Stop any other CD drive so does not crash computer
	
	Newbie = true;
		
	if(whichCD > CDROM_4)
		whichCD = CDROM_1;	//Set to default if out of bounds

	//setup CDROM
	devicefd = open((char*) DeviceNames->ItemAt(whichCD), O_RDONLY);
	if (devicefd < 0)
	{
		//log("CD Error: Could not find CD device%d.",whichCD+1);
		return false;
	}
	
	ActiveDevice = whichCD;

	return true;	
	
}

D_Cd::~D_Cd()
{
	float VolNow;
	
	VolNow = CurrentVolume[0];

	switch(GetNumDevices())		//Stop all devices
	{
		default: 
		case 4:	Init(CDROM_4);
				VolumeSet(0);		//So they will not here change
				SelectTrack(1);		//To reset cd for next time around
				Stop();				//(Some wierd bug with plain audio disks)	
		case 3:	Init(CDROM_3);
				VolumeSet(0);
				SelectTrack(1);
				Stop();
		case 2: Init(CDROM_2);
				VolumeSet(0);
				SelectTrack(1);
				Stop();		
		case 1: Init(CDROM_1);
				VolumeSet(0);
				SelectTrack(1);
				Stop();
				if(ResetVolumeLevel)		
					VolumeSet(InitialVolume);	//Restore default volume level
				else
					VolumeSet(VolNow);
			break;
	
	}
	
	// Free the names of the CD player devices.
	int32 countItems = DeviceNames->CountItems(); 
	for (int i=0; i<countItems; ++i) 
	{
		delete DeviceNames->RemoveItem((int32) 0);
    }
	delete DeviceNames;
	
	if(devicefd >= 0)
		close(devicefd);
		
	delete ParamWeb;
}


void D_Cd::PlayOrPause()
{
	// if paused, or stopped, plays, if playing, pauses
	UpdateState();
	switch (CurrentState) 
	{
		case NoCD:
			 Play();
			 break;
		case Stopped:	
			 Play();
			 break;
		case Paused:
			 if(Newbie)
			 {
			 	Play(); Newbie = false; //special hack for when new cd present
			 }
			 else		
			 	PlayContinue();
			 break;
		case Playing:	
			 Pause();
			 break;
			 
		default:
			 break;
	}
}


bool D_Cd::Pause()
{

	UpdateState();
	//if already paused then start playing again
	if(CurrentState == Paused)
	{
		PlayContinue();
		return false;
	}
	
	// pause the CD
	status_t result = ioctl(devicefd, B_SCSI_PAUSE_AUDIO);
	if (result != B_NO_ERROR) 
	{
		//log("CD Error: Could not pause CD device.");
		return false;
	}

	return true;
}

bool D_Cd::Play()
{

	// play the CD
	UpdateState();
	if (CurrentState == NoCD) 
	{
		// no CD available, bail out
		ioctl(devicefd, B_LOAD_MEDIA, 0, 0);
		//log("CD Error: Could not play CD device, CD not pressent.");
		return false;
	}
	
	scsi_play_track track;

	if(CurrentTrack < 1)		//Does not like trying to play track 0
		CurrentTrack = 1;	

	track.start_track = CurrentTrack;
	track.start_index = 1;
	track.end_track = 99;
	track.end_index = 1;

	if(IsTrackData(1))	//Skip data track
	{
		CurrentTrack = 2;
		track.start_track = 2;
	}

	status_t result = ioctl(devicefd, B_SCSI_PLAY_TRACK, &track);
	if (result != B_NO_ERROR) 
	{
		//log("CD Error: Could not play CD track.");		
		return false;
	}

	return true;
}

bool D_Cd::PlayContinue()
{
	UpdateState();

	// continue after a pause
	status_t result = ioctl(devicefd, B_SCSI_RESUME_AUDIO);
	if (result != B_NO_ERROR) 
	{
		//log("CD Error: Could not continue playing CD after pause.");
		return false;
	}
	
	return true;
	
}

bool D_Cd::Stop()
{
	// stop a playing CD
	status_t result = ioctl(devicefd, B_SCSI_STOP_AUDIO);
	if (result != B_NO_ERROR) 
	{
	//	log("CD Error: Could not stop CD device.");
		return false;
	}
	
	return true;
}

bool D_Cd::Eject()
{

#ifdef B_BEOS_VERSION_4_1
	// eject or load a CD
	status_t media_status = B_DEV_NO_MEDIA;
	
	// get the status first
	ioctl(devicefd, B_GET_MEDIA_STATUS, &media_status, sizeof(media_status));
	
	// if door open, load the media, else eject the CD
	status_t result = ioctl(devicefd,
		media_status == B_DEV_DOOR_OPEN ? B_LOAD_MEDIA : B_EJECT_DEVICE);
#else

	// eject CD
	status_t result = ioctl(devicefd, B_EJECT_DEVICE);

#endif
	if (result != B_NO_ERROR) 
	{
		//log("CD Error: Could not eject CD.");
		return false;
	}
	
	return true;
	
}

bool D_Cd::SkipOneForward()
{

	// skip forward by one track
	UpdateState();
	
	if (CurrentState == NoCD)
	{
		//log("CD Error: Could not skip one forward, CD not pressent.");
		return false;
	}

	//bool wasPaused = CurrentState == Paused
	//	|| CurrentState == Stopped;

	if(CurrentTrack < NumTracks)
	{
		
		CurrentTrack += 1;
		if(CurrentTrack == (NumTracks) && IsTrackData(NumTracks)) //Fix for the damn enhanced cds (last track is data on these type) shows up as track 0 though !?!
			CurrentTrack = 1;
		
		SelectTrack(CurrentTrack);
	}
	else
	{
		CurrentTrack = 1;
		SelectTrack(CurrentTrack);
	}
	
	//if (wasPaused)
		// make sure we don't start playing if we were paused before
	//	Pause();
	
	return true;	
		
}

bool D_Cd::SkipOneBackward()
{

	// skip backward by one track
	UpdateState();
		
	if (CurrentState == NoCD)
	{
		//log("CD Error: Could not skip one backward, CD not pressent.");
		return false;
	}
	
	//bool wasPaused = CurrentState == Paused
	//	|| CurrentState == Stopped;

	if(CurrentTrack==2 && IsTrackData(1))	//Skip data track
	{
		CurrentTrack = NumTracks;
		SelectTrack(CurrentTrack);		
	}
	else if(CurrentTrack > 1)
	{
		CurrentTrack -= 1;	
		SelectTrack(CurrentTrack);
	}
	else
	{
		if(!IsTrackData(NumTracks))		
			CurrentTrack = NumTracks;
		else
			CurrentTrack = NumTracks-1;	//Fix for the damn enhanced cds (last track is data on these type) shows up as track 0 though !?!
		SelectTrack(CurrentTrack);
	}

	//if (wasPaused)
		// make sure we don't start playing if we were paused before
	//	Pause();

	return true;

}

bool D_Cd::StartScaningBackward()
{
	// start skipping
	UpdateState();
		
	if (CurrentState == NoCD)
	{
		//log("CD Error: Could not start skipping backward, CD not pressent.");	
		return false;
	}
	
	scsi_scan scan;
	scan.direction = -1;
	scan.speed = 1;
	status_t result = ioctl(devicefd, B_SCSI_SCAN, &scan);
	if (result != B_NO_ERROR) 
	{
		//log("CD Error: Could not skip track backward.");
		return false;
	}

	return true;

}


bool D_Cd::StartScaningForward()
{
	// start skipping
	UpdateState();
	
	if (CurrentState == NoCD)
	{
		//log("CD Error: Could not start skipping forward, CD not pressent.");	
		return false;
	} 

	scsi_scan scan;
	scan.direction = 1;
	scan.speed = 1;
	status_t result = ioctl(devicefd, B_SCSI_SCAN, &scan);
	if (result != B_NO_ERROR) 
	{
		//log("CD Error: Could not skip track forward.");
		return false;
	}

	return true;
	
}

bool D_Cd::StopScaning()
{
	// stop skipping
	UpdateState();
	
	if (CurrentState == NoCD)
	{
		//log("CD Error: Could not stop skipping, CD not pressent.");	
		return false;
	}

	status_t result = ioctl(devicefd, B_SCSI_RESUME_AUDIO);
	if (result != B_NO_ERROR) 
	{
		//log("CD Error: Could not resume audio after skipping track.");
		return false;
	}

	scsi_scan scan;
	scan.direction = 0;
	scan.speed = 1;
	result = ioctl(devicefd, B_SCSI_SCAN, &scan);
	if (result != B_NO_ERROR) 
	{
		//log("CD Error: Could not stop skipping track.");
		return false;
	}
	
	return true;

}

bool D_Cd::SelectTrack(int trackNumber)
{	
	// go to a selected track
	UpdateState();

	if (CurrentState == NoCD)
	{
		//log("CD Error: Could not select track, CD not pressent.");	
		return false;
	}

	scsi_play_track track;

	if(trackNumber < 1)		//Does not like trying to play track 0
	{
		CurrentTrack = 1;	
		trackNumber = 1;	
	}

	if(trackNumber < 2 && IsTrackData(1))	//Skip data track
	{
		CurrentTrack = 2;
		trackNumber = 2;
	}
	
	track.start_track = trackNumber;
	track.start_index = 1;
	track.end_track = 99;
	track.end_index = 1;

	status_t result = ioctl(devicefd, B_SCSI_PLAY_TRACK, &track);
	if (result != B_NO_ERROR) 
	{
		//log("CD Error: Could not play after selecting track.");	
		return false;
	}

	CurrentTrack = trackNumber;
	return true;

}

int D_Cd::FindCDDevices(const char *directory)
{ 

	BDirectory dir; 
	dir.SetTo(directory); 
	if(dir.InitCheck() != B_NO_ERROR) 
	{ 
		//log("CD Error: Could not find directory where CD-ROMs are located.");
		return false; 
	} 
	dir.Rewind(); 
	BEntry entry; 
	while(dir.GetNextEntry(&entry) >= 0) 
	{ 
		BPath path; 
		const char *name; 
		entry_ref e; 
		
		if(entry.GetPath(&path) != B_NO_ERROR) 
			continue; 
		name = path.Path(); 
		
		
		if(entry.GetRef(&e) != B_NO_ERROR) 
			continue; 

		if(entry.IsDirectory()) 
		{ 
			if(strcmp(e.name, "floppy") == 0) 
				continue; // ignore floppy (it is not silent) 

			int devfd = FindCDDevices(name);
			if(devfd >= 0)
				return devfd;
		} 
		else 
		{ 
			int devfd; 
			device_geometry g; 

			if(strcmp(e.name, "raw") != 0) 
				continue; // ignore partitions 

			devfd = open(name, O_RDONLY); 

			if(devfd < 0) 
				continue; 

			if(ioctl(devfd, B_GET_GEOMETRY, &g, sizeof(g)) >= 0) 
			{
				if(g.device_type == B_CD)
				{ 	
				    // Store the device's name in the list and close the device. 
				    DeviceNames->AddItem(strdup(name));				
					//return devfd;
				}
			}
			close(devfd);
		} 
	}//end while
	return B_ERROR;

}

void D_Cd::UpdateState()
{
	// check the current CD play state and force a notification to
	// be sent if it changed from last time
	NumTracks = GetNumTracks();
	
	scsi_position pos;
	status_t media_status = B_DEV_NO_MEDIA;
	ioctl(devicefd, B_GET_MEDIA_STATUS, &media_status, sizeof(media_status));
	if (media_status != B_NO_ERROR)
		CurrentState = NoCD;
		
	status_t result = ioctl(devicefd, B_SCSI_GET_POSITION, &pos);

	if (result != B_NO_ERROR)
		CurrentState = NoCD;
		
	else if ((!pos.position[1]) || (pos.position[1] >= 0x13) ||
	   ((pos.position[1] == 0x12) && (!pos.position[6])))
		CurrentState = Stopped;
		
	else if (pos.position[1] == 0x11)
		CurrentState = Playing;
	else
		CurrentState = Paused;

	// check the current CD track number		
	if (!pos.position[1] || pos.position[1] >= 0x13
		|| (pos.position[1] == 0x12 && !pos.position[6]))
		 CurrentTrack = 0;
	else
		 CurrentTrack = pos.position[6];

}

int D_Cd::GetNumTracks()
{
	// get the number of tracks on the current CD
	UpdateTOC();

// DOES NOT SEEM TO WORK ANYMORE (tried with Metallica S&M)	
	//	if(IsTrackData(NumTracks))
	//		return (tableOfContents.lastTrack - 1);	//take care of enhanced CDs
	//	else
		return tableOfContents.lastTrack;
	
}

CDState D_Cd::GetCurrentState()
{
	return CurrentState;
}

void D_Cd::VolumeUp()
{

	// support stereo (2-channel) controls
	float chanData[2];
	size_t	datasize;

	if(SoundCardPresent && CanChangeCDVolume)
	{
		// figure out what "now" is in the performance time base associated
		// with the node that the BParameterWeb controls
		BMediaRoster* roster = BMediaRoster::Roster();
		BTimeSource* timeSource = roster->MakeTimeSourceFor(ParamWeb->Node());
		bigtime_t now = timeSource->Now();

		datasize = sizeof(chanData);
	
		// now set the BParameter!
		GainParam->GetValue(&chanData, &datasize, &now);
		// must delete the time source from BMediaRoster::MakeTimeSourceFor(...)
		timeSource->Release();			
			
		// transform the control values into their proper ranges - this is always
		// a linear mapping from the control's range to the parameter's

		if(chanData[0] < (MaxVolume - StepValue) )//&& chanData[1] < MaxVolume/*-1*/)
		{
			chanData[0] = chanData[0]/*CurrentVolume[0]*/ + StepValue;
			chanData[1] = chanData[1]/*CurrentVolume[0]*/ + StepValue;
		}
		else
		{
			chanData[0] = MaxVolume;
			chanData[1] = MaxVolume;							
		}

		CurrentVolume[0] = chanData[0];		//Set global for future use
		CurrentVolume[1] = chanData[1];		//Set global for future use	

		// figure out what "now" is in the performance time base associated
		// with the node that the BParameterWeb controls
		roster = BMediaRoster::Roster();
		timeSource = roster->MakeTimeSourceFor(ParamWeb->Node());
		now = timeSource->Now();

		// now set the BParameter!
		GainParam->SetValue(&chanData, sizeof(chanData), now);

		// must delete the time source from BMediaRoster::MakeTimeSourceFor(...)
		timeSource->Release();

	}//end if SoundCardPresent

}

void D_Cd::VolumeDown()
{

	// support stereo (2-channel) controls
	float chanData[2];
	size_t	datasize;

	if(SoundCardPresent && CanChangeCDVolume)
	{
		// figure out what "now" is in the performance time base associated
		// with the node that the BParameterWeb controls
		BMediaRoster* roster = BMediaRoster::Roster();
		BTimeSource* timeSource = roster->MakeTimeSourceFor(ParamWeb->Node());
		bigtime_t now = timeSource->Now();

		datasize = sizeof(chanData);
	
		// now set the BParameter!
		GainParam->GetValue(&chanData, &datasize, &now);
		// must delete the time source from BMediaRoster::MakeTimeSourceFor(...)
		timeSource->Release();			
			
		// transform the control values into their proper ranges - this is always
		// a linear mapping from the control's range to the parameter's

		if(chanData[0] > (MinVolume + StepValue)/*+1*/ )//&& chanData[1] > MinVolume/*+1*/)
		{
			chanData[0] = chanData[0]/*CurrentVolume[0]*/ - StepValue;
			chanData[1] = chanData[1]/*CurrentVolume[0]*/ - StepValue;		
		}
		else
		{
			chanData[0] = MinVolume;
			chanData[1] = MinVolume;									
		}

		CurrentVolume[0] = chanData[0];		//Set global for future use
		CurrentVolume[1] = chanData[1];		//Set global for future use	

		// figure out what "now" is in the performance time base associated
		// with the node that the BParameterWeb controls
		roster = BMediaRoster::Roster();
		timeSource = roster->MakeTimeSourceFor(ParamWeb->Node());
		now = timeSource->Now();

		// now set the BParameter!
		GainParam->SetValue(&chanData, sizeof(chanData), now);

		// must delete the time source from BMediaRoster::MakeTimeSourceFor(...)
		timeSource->Release();
		
	}//end if SoundCardPresent

}

void D_Cd::VolumeSet(float level)
{
	// support stereo (2-channel) controls
	float chanData[2];
			
	// transform the control values into their proper ranges - this is always
	// a linear mapping from the control's range to the parameter's

	if(SoundCardPresent && CanChangeCDVolume)
	{
		chanData[0] = level;
		chanData[1]	= level;

		if(level > MaxVolume)
		{
			chanData[0] = MaxVolume;
			chanData[1]	= MaxVolume;
		}

		if(level < MinVolume)
		{
			chanData[0] = MinVolume;
			chanData[1]	= MinVolume;
		}

		CurrentVolume[0] = chanData[0];		//Set global for future use
		CurrentVolume[1] = chanData[1];		//Set global for future use	

		// figure out what "now" is in the performance time base associated
		// with the node that the BParameterWeb controls
		BMediaRoster* roster = BMediaRoster::Roster();
		BTimeSource* timeSource = roster->MakeTimeSourceFor(ParamWeb->Node());
		bigtime_t now = timeSource->Now();

		// now set the BParameter!
		GainParam->SetValue(&chanData, sizeof(chanData), now);

		// must delete the time source from BMediaRoster::MakeTimeSourceFor(...)
		timeSource->Release();

	}//end if SoundCardPresent

}

float D_Cd::GetCurrentVolume()
{
	return CurrentVolume[0];
}

float D_Cd::GetMaxVolume()
{
	return MaxVolume;
}

float D_Cd::GetMinVolume()
{
	return MinVolume;
}

void D_Cd::ResetVolume(int flag)
{
	if(flag)
		ResetVolumeLevel = true;
	else
		ResetVolumeLevel = false;
}

int D_Cd::GetNumDevices()
{
	return DeviceNames->CountItems();
}

int D_Cd::GetCurrentTrack()
{
	UpdateState();
	return CurrentTrack;
}

int D_Cd::UpdateTOC()
{

	status_t result = ioctl(devicefd, B_SCSI_GET_TOC, &tableOfContents);
	
	if (result != B_NO_ERROR)
	{
	//	log("CD Error: Could not get disks Table of Contents.");
		return false;
	}

	return true;
	
}

int D_Cd::CurrentTrackMin()
{
	int	temp_min = tableOfContents.track[CurrentTrack-1].address.minute;	

	scsi_position pos;

	status_t result =ioctl(devicefd, B_SCSI_GET_POSITION, &pos);
	if (result != B_NO_ERROR)
		return -1;
	else if ((!pos.position[1]) || (pos.position[1] >= 0x13) ||
	   ((pos.position[1] == 0x12) && (!pos.position[6])))
		return 0;
	else
	{
		return (pos.position[9] - temp_min);
	}
}

int D_Cd::CurrentTrackSec()
{
	int	temp_sec = tableOfContents.track[CurrentTrack-1].address.second;

	scsi_position pos;

	status_t result =ioctl(devicefd, B_SCSI_GET_POSITION, &pos);
	if (result != B_NO_ERROR)
		return -1;
	else if ((!pos.position[1]) || (pos.position[1] >= 0x13) ||
	   ((pos.position[1] == 0x12) && (!pos.position[6])))
		return 0;
	else
		return (pos.position[10] - temp_sec);

}

int D_Cd::CurrentTotalMin()
{

	scsi_position pos;

	status_t result =ioctl(devicefd, B_SCSI_GET_POSITION, &pos);
	if (result != B_NO_ERROR)
		return -1;
	else if ((!pos.position[1]) || (pos.position[1] >= 0x13) ||
	   ((pos.position[1] == 0x12) && (!pos.position[6])))
		return 0;
	else
		if(IsTrackData(1))	//Subract time data track reports (just want audio)
			return pos.position[9] - tableOfContents.track[1].address.minute;
		else
			return pos.position[9];			
}

int D_Cd::CurrentTotalSec()
{

	scsi_position pos;

	status_t result =ioctl(devicefd, B_SCSI_GET_POSITION, &pos);
	if (result != B_NO_ERROR)
		return -1;
	else if ((!pos.position[1]) || (pos.position[1] >= 0x13) ||
	   ((pos.position[1] == 0x12) && (!pos.position[6])))
		return 0;
	else
		if(IsTrackData(1))	//Subract time data track reports (just want audio)
			return pos.position[10] - tableOfContents.track[1].address.second;
		else
			return pos.position[10];	

}

int	D_Cd::CurrentTrackMinLeft()	
{											

	scsi_position pos;

	status_t result =ioctl(devicefd, B_SCSI_GET_POSITION, &pos);
	if (result != B_NO_ERROR)
		return -1;
	else if ((!pos.position[1]) || (pos.position[1] >= 0x13) ||
	   ((pos.position[1] == 0x12) && (!pos.position[6])))
		return 0;
	else
		return tableOfContents.track[CurrentTrack].address.minute - pos.position[9];
					
}
	
int D_Cd::CurrentTrackSecLeft()		
{													
	scsi_position pos;

	status_t result =ioctl(devicefd, B_SCSI_GET_POSITION, &pos);
	if (result != B_NO_ERROR)
		return -1;
	else if ((!pos.position[1]) || (pos.position[1] >= 0x13) ||
	   ((pos.position[1] == 0x12) && (!pos.position[6])))
		return 0;
	else
		return tableOfContents.track[CurrentTrack].address.second - pos.position[10];

}

int	D_Cd::CurrentTotalMinLeft()		
{

	scsi_position pos;

	status_t result =ioctl(devicefd, B_SCSI_GET_POSITION, &pos);
	if (result != B_NO_ERROR)
		return -1;
	else if ((!pos.position[1]) || (pos.position[1] >= 0x13) ||
	   ((pos.position[1] == 0x12) && (!pos.position[6])))
		return 0;
	else
		if(IsTrackData(NumTracks))	//Count for enhanced CDs
			return tableOfContents.track[NumTracks-1].address.minute - pos.position[9];
		else
			return tableOfContents.track[NumTracks].address.minute - pos.position[9];

}

int	D_Cd::CurrentTotalSecLeft()		
{

	scsi_position pos;

	status_t result =ioctl(devicefd, B_SCSI_GET_POSITION, &pos);
	if (result != B_NO_ERROR)
		return -1;
	else if ((!pos.position[1]) || (pos.position[1] >= 0x13) ||
	   ((pos.position[1] == 0x12) && (!pos.position[6])))
		return 0;
	else
		if(IsTrackData(NumTracks))	//Count for enhanced CDs
			return tableOfContents.track[NumTracks-1].address.second - pos.position[10];
		else
			return tableOfContents.track[NumTracks].address.second - pos.position[10];

}

bool D_Cd::IsTrackData(int tracknum)
{
	//Check if specified track number is data or audio (usually check track 1)
	UpdateTOC();
	
		//tracknun-1 because us programers like to start counting at 0
	if((tableOfContents.track[tracknum-1].flags & FLAG_DATA_TRACK) != 0)
	{
		return true;
	}

	return false;
	
}