#ifndef __CD_H__
#define __CD_H__

//Be Specific headers
#include <ParameterWeb.h>
#include <TimeSource.h>
#include <MediaRoster.h>
#include <Application.h>
#include <Directory.h>
#include <Path.h>
#include <List.h>

//Other headers
#include <scsi.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>


#define CDROM_1		0
#define CDROM_2		1
#define CDROM_3		2
#define CDROM_4		3

#define FLAG_DATA_TRACK 	0x04

extern FILE 			*logfile;			//Error log
extern void log(const char *fmt, ...);

enum CDState 
{
	NoCD,
	Stopped,
	Paused,
	Playing,
	Skipping
};

typedef struct Address
{
	uint8 reserved;
	uint8 minute;
	uint8 second;
	uint8 frame;
	
};

/** An entry in the table of contents. */
typedef struct Track
{
	uint8   reserved1;
	uint8   flags;
	uint8   trackNumber;
	uint8   reserved2;
	Address address;
};

/** The table of contents for an entire CD. */
typedef struct TableOfContents
{
	uint16 length;      // Number of bytes in the table of contents.  
	uint8  firstTrack;  // The track number which will appear in track[0].
	uint8  lastTrack;   // The last valid entry in the table of contents.
	Track  track[100];
};

class D_Cd
{

public:

			D_Cd(int initialDevice);
	virtual ~D_Cd();
	
	bool 	Init(int whichCD);			//Initializes the specified CDROM 1 - 4
	int 	FindCDDevices(const char *directory);
										//Finds the CD drives

	void 	PlayOrPause();				//Play is paused or pause if playing
	bool 	PlayContinue();				//Continue playing after pause
	bool	Play();						//Play the current track
	bool	Pause();					//Pause the CD
	bool 	Stop();						//Stop the CD
	bool 	Eject();					//Spit the CD out
	bool 	SkipOneForward();			//Change to the next track
	bool 	SkipOneBackward();			//Change to the previous track
	bool 	StartScaningBackward();
	bool	StartScaningForward();		
	bool 	StopScaning();				//Stop the track skipping
	bool	SelectTrack(int trackNumber);//Select track to play
	void 	UpdateState();				//Gets current state of the CDROM

	int		GetNumDevices();			//Returns the number of CD drives in system
	int 	GetNumTracks();				//Returns the number of tracks on CD
	int		GetCurrentTrack();			//Returns the current track being played
	CDState	GetCurrentState();			//Returns the current state of device
	float	GetCurrentVolume();			//Returns the current CD volume (does not return both chanels separately)
	float	GetMaxVolume();				//Maximum volume level sound card supports
	float	GetMinVolume();				//Minimum volume level sound card supports
	bool 	IsTrackData(int tracknum);	//Returns true if track is data not audio
	
	void 	VolumeUp();					//Raise volume level by 1
	void 	VolumeDown();				//Lower volume level by 1
	void 	VolumeSet(float level);		//Set volume level
	
	void	ResetVolume(int flag);		//Toggles whether or not to reset original volume on exit
	
	int		UpdateTOC();				//Updates disk Table of Contents
	int		CurrentTotalMin();			//Total amount of minutes played
	int		CurrentTotalSec();			//Total amount of seconds played
	int 	CurrentTrackMin();			//How many minutes have played on current track
	int 	CurrentTrackSec();			//How many seconds have played on current track	

	int		CurrentTrackMinLeft();		//How many min left of current track
	int 	CurrentTrackSecLeft();		//How many sec left of current track
	int		CurrentTotalMinLeft();		//Total min left to play on disk
	int		CurrentTotalSecLeft();		//Total sec left to play on disk
		
	int		CurrentActiveDevice(){return ActiveDevice+1;}		
		
		
private:

	TableOfContents tableOfContents;

	BList		*DeviceNames;			//List of CDROM device paths up to 4

	float		InitialVolume;			//What CD volume was before messing with it

	bool		SoundCardPresent;		//Flag to indicate if should try changing volume level
	bool		ResetVolumeLevel;
	bool        CanChangeCDVolume;
	
	int 		devicefd;				//Device number?
	
	CDState		CurrentState;			//The state of the CDROM
	float 		CurrentVolume[2];		//The current cd volume (0 = left and 1 = right, I think)
	int			CurrentTrack;			//The track currently selected for play or currently playing		
	int			NumTracks;				//Number of tracks on the CD

	float 		MaxVolume;				//Maximum allowed CD volume
	float		MinVolume;				//Minimum allowed CD volume
	
	float 		StepValue;				//How much to increment volume by (dictated by sound card)

	int			ActiveDevice;
	bool		Newbie;					//Flag for using PlayOrPause when new disk present

	BParameterWeb *ParamWeb;			//Used to get the current volume from the media controls
	BContinuousParameter *GainParam;	//and used in changing those values


};


#endif