// Keyboard input stuff for the BeOS

#ifndef INPUT_H
#define INPUT_H

#include <Window.h>
#include <InterfaceDefs.h>		//for get_key_info() in BeOS
#include <Joystick.h>

//#include "main.h"
//#include "states.h"

#include <malloc.h>


extern BWindow			*handler;	//Handle to window

//Mouse codes for BeOS
#define D_MOUSE1		  B_PRIMARY_MOUSE_BUTTON
#define D_MOUSE2		  B_SECONDARY_MOUSE_BUTTON
#define D_MOUSE3		  B_TERTIARY_MOUSE_BUTTON

//Joystick codes
#define D_JOY_1			  0
#define D_JOY_2			  1
#define D_JOY_3			  2
#define D_JOY_4			  3
#define D_JOY_5			  4
#define D_JOY_6			  5
#define D_JOY_7			  6
#define D_JOY_8			  7
// can go up to 32 buttons max

#define D_JOY_AXIS_X	  0
#define D_JOY_AXIS_Y	  1
#define D_JOY_AXIS_Z	  2
// can support an unspecified amount of axes

#define D_JOY_HAT_CENTERED  	0
#define D_JOY_HAT_UP	  		1
#define D_JOY_HAT_UP_RIGHT		2
#define D_JOY_HAT_RIGHT			3
#define D_JOY_HAT_DOWN_RIGHT	4
#define D_JOY_HAT_DOWN			5
#define D_JOY_HAT_DOWN_LEFT		6
#define D_JOY_HAT_LEFT			7
#define D_JOY_HAT_UP_LEFT		8

//These are a little different in BeOS
#define D_ESCAPE          0x01
#define D_1               0x12
#define D_2               0x13
#define D_3               0x14
#define D_4               0x15
#define D_5               0x16
#define D_6               0x17
#define D_7               0x18
#define D_8               0x19
#define D_9               0x1A
#define D_0               0x1B
#define D_MINUS           0x1C    /* - on main keyboard */
#define D_EQUALS          0x1D
#define D_BACK            0x1E    /* backspace */
#define D_TAB             0x26
#define D_Q               0x27
#define D_W               0x28
#define D_E               0x29
#define D_R               0x2A
#define D_T               0x2B
#define D_Y               0x2C
#define D_U               0x2D
#define D_I               0x2E
#define D_O               0x2F
#define D_P               0x30
#define D_LBRACKET        0x31
#define D_RBRACKET        0x32
#define D_RETURN          0x47    /* Enter on main keyboard */
#define D_LCONTROL        0x5C
#define D_A               0x3C
#define D_S               0x3D
#define D_D               0x3E
#define D_F               0x3F
#define D_G               0x40
#define D_H               0x41
#define D_J               0x42
#define D_K               0x43
#define D_L               0x44
#define D_SEMICOLON       0x45
#define D_APOSTROPHE      0x46
#define D_GRAVE           0x11    /* accent grave */
#define D_LSHIFT          0x4B
#define D_BACKSLASH       0x33
#define D_Z               0x4C
#define D_X               0x4D
#define D_C               0x4E
#define D_V               0x4F
#define D_B               0x50
#define D_N               0x51
#define D_M               0x52
#define D_COMMA           0x53
#define D_PERIOD          0x54    /* . on main keyboard */
#define D_SLASH           0x55    /* / on main keyboard */
#define D_RSHIFT          0x56
#define D_MULTIPLY        0x24    /* * on numeric keypad */
#define D_LMENU           0x5D    /* left Alt */
#define D_SPACE           0x5e
#define D_CAPITAL         0x3B	  //is this caps lock??
#define D_F1              0x02 
#define D_F2              0x03
#define D_F3              0x04
#define D_F4              0x05
#define D_F5              0x06
#define D_F6              0x07
#define D_F7              0x08
#define D_F8              0x09
#define D_F9              0x0A
#define D_F10             0x0B
#define D_NUMLOCK         0x22
#define D_SCROLL          0x0F    /* Scroll Lock */
#define D_NUMPAD7         0x37
#define D_NUMPAD8         0x38
#define D_NUMPAD9         0x39
#define D_SUBTRACT        0x25    /* - on numeric keypad */
#define D_NUMPAD4         0x48
#define D_NUMPAD5         0x49
#define D_NUMPAD6         0x4A
#define D_ADD             0x3A    /* + on numeric keypad */
#define D_NUMPAD1         0x58
#define D_NUMPAD2         0x59
#define D_NUMPAD3         0x5A
#define D_NUMPAD0         0x64
#define D_DECIMAL         0x65    /* . on numeric keypad */
#define D_F11			  0x0C
#define D_F12			  0x0D

#define D_NUMPADENTER     0x5B    /* Enter on numeric keypad */
#define D_RCONTROL        0x60
//#define D_NUMPADCOMMA     0x5B    /* , on numeric keypad (NEC PC98) */
#define D_DIVIDE          0x23    /* / on numeric keypad */
#define D_SYSRQ           0x7E
#define D_RMENU           0x5F    /* right Alt */
#define D_HOME            0x0F    /* Home on arrow keypad */
#define D_UP              0x57    /* UpArrow on arrow keypad */
#define D_PRIOR           0x7F    /* PgUp on arrow keypad */
#define D_LEFT            0x61    /* LeftArrow on arrow keypad */
#define D_RIGHT           0x63    /* RightArrow on arrow keypad */
#define D_END             0x0F    /* End on arrow keypad */
#define D_DOWN            0x62    /* DownArrow on arrow keypad */
#define D_NEXT            0x10    /* PgDn on arrow keypad */
#define D_INSERT          0x7E    /* Insert on arrow keypad */
#define D_DELETE          0x0E    /* Delete on arrow keypad */
#define D_LWIN            0x66    /* Left Windows key */
#define D_RWIN            0x67    /* Right Windows key */
#define D_APPS            0x68    /* AppMenu key */


class KEYBOARD
{
	
	public:
							KEYBOARD();
			void 			Update();	
			int				KeyDown(int key);
			key_info		KeyBuffer;		//Holds the key states
			
			//The below functions are for debouncing
			uint			states[128];

			bool 			Check(uint key);
			void  			MakeDown(uint key);
			void			MakeUp(uint key);
			void 			Init();
			
};//end keyboard class

class MOUSE
{

	public:
			uint32			Button;		//Curent pressed button
			BPoint			WinPos;		//Position of mouse within window
			BPoint			ScreenPos;	//Position of mouse relative to whole screen
			void 			Update();	

};//end mouse class


class JOYSTICK
{
	public:
		
							JOYSTICK();
			void			Update();
		
			int32			NumSticks;		//Number of Joysticks
			int32			NumAxes;		//Total number of axes present
			int32 			NumButtons;		//Total number of buttons
			int32			NumHats;		//Total number of hats
			int32			NumDevices;		//Total number of game ports 
			
			int16			XPosition;		//Current X position axis 0
			int16			YPosition;		//Current Y position axis 1
			int16			ZPosition;		//Current Z position axis 2
		
			uint32			ButtonStates;	//Holds the states of all the buttons	
			int16			*AxesStates;	//Holds the states of all the axes
			uint8			*HatStates;		//Holds the states of all the hats
	
	private:
			
			BJoystick		JoyStick;			

};//end joystick class



//Global Input variables
extern KEYBOARD		Keyboard;		
extern MOUSE		Mouse;			//MOUSE INFO STRUCTURE
extern JOYSTICK		Joystick;

//Macros
//#define KEY_DOWN(key)				Keyboard.KeyBuffer.key_states[key >> 3] & (1 << (7 - (key % 8)))
#define JOY_BUTTON_DOWN(button)		(Joystick.ButtonStates & (1 << button))
#define JOY_HAT_DOWN(button)		(Joystick.HatStates[button])


extern bool MOUSE_DOWN(uint32 button);
extern bool KEY_DOWN(int key);

#endif // _INPUT_H









