#include <Application.h>
#include <Window.h>
#include <View.h>
#include <Alert.h>

#include <Bitmap.h>					//for skinning app
#include <TranslationUtils.h>
#include <Screen.h>

#include <FindDirectory.h>
#include <Directory.h>
#include <Entry.h>
#include <stdio.h>
#include <stdlib.h>
#include <Roster.h>
#include <Invoker.h>

#include "App.h"
#include "AppWin.h"
#include "AppView.h"
#include "SetWindow.h"
#include "Constants.h"
#include "MsgVals.h"

#include "states.h"
#include "cd.h"

//GLOBALS
SetWindow		*setWindow;			//window for settings
D_Cd			*cd;				//CD engine
FILE 			*logfile;			//Error log
D_STATES		states;				//Global settings states	


//Source of individual letters in bitmap
BRect		A_Rect(0,3,4,8),			B_Rect(5,3,9,8),
			C_Rect(10,3,14,8),			D_Rect(15,3,19,8),
			E_Rect(20,3,24,8),			F_Rect(25,3,29,8),
			G_Rect(30,3,34,8),			H_Rect(35,3,39,8),
			I_Rect(40,3,44,8),			J_Rect(45,3,49,8),
			K_Rect(50,3,54,8),			L_Rect(55,3,59,8),
			M_Rect(60,3,64,8),			N_Rect(65,3,69,8),
			O_Rect(70,3,74,8),			P_Rect(75,3,79,8),
			Q_Rect(80,3,84,8),			R_Rect(85,3,89,8),
			S_Rect(90,3,94,8),			T_Rect(95,3,99,8),
			U_Rect(100,3,104,8),		V_Rect(105,3,109,8),
			W_Rect(110,3,114,8),		X_Rect(115,3,119,8),
			Y_Rect(120,3,124,8),		Z_Rect(125,3,129,8),
			Minus_Rect(75,9,79,14);				


BRect		SkipBKWDDest(0,24,20,41),		//rectangle bounds of buttons in app
			PlayDest(21,24,41,41),
			PauseDest(42,24,62,41),
			StopDest(63,24,83,41),
			SkipFWDDest(84,24,104,41),
			EjectDest(105,24,125,41),
			ShuffleDest(126,6,146,23),	
			RepeatDest(126,24,146,41),
			VolDownDest(147,24,167,41),
			VolUpDest(147,6,167,23),		
			CD1Dest(97,1,109,10),
			CD2Dest(112,1,124,10),
			CD3Dest(97,13,109,22),
			CD4Dest(112,13,124,22),
			About_SetDest(126,0,167,5),	//Destination to draw about & settings button			
		    AboutDest(126,0,152,5),		//About button			
		    SettingsDest(153,0,167,5),	//Settings Window
			CDPosDest(96,0,125,23),
			IndicatorPosDest(47,4,54,9),			
			TrackDisplayDest(0,0,95,11),
			TimeDisplayDest(58,0,95,11),		
			VolumeDisplayDest(0,12,95,23),	//Volume display background
			VolPosDest(3,16,7,21),		//Start of where to place volume level
			TrackPosDest(3,4,7,9),		//Start of where to place track num
			TimePosDest(71,4,75,9);		//Start of where to place track time
										//in app cords

BRect		SkipBKWDDownSrc(0,53,20,70),		//rectangle bounds of buttons in bitmap
			PlayDownSrc(22,53,42,70),
			PauseDownSrc(44,53,64,70),
			StopDownSrc(66,53,86,70),
			SkipFWDDownSrc(88,53,108,70),
			EjectDownSrc(110,53,130,70),
			ShuffleDownSrc(132,53,152,70),	
			RepeatDownSrc(154,53,174,70),
			VolDown_DownSrc(176,53,196,70),
			VolUp_DownSrc(198,53,218,70),
			
			SkipBKWDUpSrc(0,35,20,52),		//rectangle bounds of buttons in bitmap
			PlayUpSrc(22,35,42,52),
			PauseUpSrc(44,35,64,52),
			StopUpSrc(66,35,86,52),
			SkipFWDUpSrc(88,35,108,52),
			EjectUpSrc(110,35,130,52),
			ShuffleUpSrc(132,35,152,52),	
			RepeatUpSrc(154,35,174,52),
			VolDown_UpSrc(176,35,196,52),
			VolUp_UpSrc(198,35,218,52),
								
			DisplaySrc(0,22,95,33),
			TimeDisplaySrc(95,22,132,33),
			CDChoiceSrc(156,0,185,23),
			AboutSrc(134,28,175,33),	//About button	
			PlayIndicatorSrc(177,25,184,32),
			PauseIndicatorSrc(186,25,193,32),
			StopIndicatorSrc(194,25,201,32);

BRect		NumberRect[10];				//Number bounds in bitmap cords
			
			
BBitmap		*ButtonsBitmap;				//bitmap of buttons


bool PlayPressed		= false;		//Used so CD will not start playing automatically on startup when shuffle mode is on by default
bool StartScaningFWD    = false; 
bool StartScaningBKWD   = false;
bool ShuffleCD		    = false;
bool LoopCD			    = false;
bool Skipped			= false;
bool Registered			= true;		//Is app registered
unsigned int nag_time_up = 0;			//Random number of seconds between 2 & 4 minutes for when nag screen should show up
unsigned int nag_count  = 0;			//Counter (in sec) used to throw up nag screen to register		 	
bool nag_done			= true;

int	 lastTrack			=0;
int	 TrackTimeType=0;	//Way to display track time	0=amount played per track
						//							1=total amount played
						//							2=amount left per track
						//							3=amount left total

int			track,minute,second=0;
CDState		state;


char 	AppPath[B_PATH_NAME_LENGTH+B_FILE_NAME_LENGTH];
char	AppLeaf[B_FILE_NAME_LENGTH];

const char  *app_signature = "application/x-vnd.Hailstone-DiscJockey";
BWindow		*handler;			//Global handle to the window

void log(const char *fmt, ...);
int LoadBitmap(const char *file,BBitmap **pic);
bool LoadConfigurationFile(char *filename);
bool SaveConfigurationFile(char *filename);
bool ReadKeyFile();


void log(const char *fmt, ...)
{
    static char buffer[1024];
	int rv = 0;
	char *argptr;


    va_start(argptr, fmt);
    rv = vsprintf( buffer, fmt, argptr );
    va_end( argptr );

	if(logfile)
	{
		fprintf(logfile,"%s\n", buffer);
		fflush(logfile);
	}
}


App::App()
		  : BApplication(app_signature)
{

	//setup log file
	BEntry			entry;	
	BPath			path;
	app_info		info;	
	unsigned int	loop=0;

	srand((unsigned) time(NULL));		//for random track play
	
	be_app->GetAppInfo(&info);
	entry.SetTo(&info.ref);
	entry.GetPath(&path);
	strcpy(AppPath,path.Path());
	strcpy(AppLeaf,path.Leaf());

	for(loop=0;loop < strlen(AppPath)-strlen(AppLeaf);loop++)
		AppPath[loop] = AppPath[loop];
	
	AppPath[loop] = (char)NULL;


	char 	*logfilepath = new char[strlen(AppPath)+strlen("CDlog")+1];
	strcpy(logfilepath,AppPath);
	strcat(logfilepath,"CDlog");
	logfile=fopen(logfilepath,"wb");	
	delete logfilepath;

	
	aWindow = new AppWindow(BRect(0,0,167,41));

	//setup number rect array
	for(loop=0;loop < 10;loop++)
	{	
		NumberRect[loop].Set(loop*5,9,(loop*5)+4,14);
	}

	
	return;
}


void App::RefsReceived(BMessage *message)
{
	switch(message->what)
	{
		case B_REFS_RECEIVED:
			setWindow->PostMessage(message);	
			break;
	}
	
}


bool App::QuitRequested()
{
	delete ButtonsBitmap;	
	
	PostMessage(B_QUIT_REQUESTED);
	return(true);
}

////////////////////////////////////////////////////////////////

AppWindow::AppWindow(BRect frame)
			: BWindow(frame, "DiscJockey",B_FLOATING_WINDOW_LOOK, B_NORMAL_WINDOW_FEEL,
					   B_NOT_ZOOMABLE | B_NOT_RESIZABLE, B_CURRENT_WORKSPACE )

{									  	
	

	//Initialize global settings
	if(LoadConfigurationFile("config.file") ==false)
	{
		states.TopLeft.x       	 	  = 5;
		states.TopLeft.y 	     	  = 5;
		states.DontStopCDWhenSwitching = 0;
		states.ShuffleAcross	 	  = 0;
		states.SaveScreenPosition	  = 1;
		states.ResetVolume			  = 1;
		states.ShuffleDefault		  = 0;
		states.LoopDefault			  = 0;
		states.DefaultDevice		  = 1;
		states.DefaultTimeMode		  = 0;
		strcpy(states.SkinPath,AppPath);
		states.VolumeColor.red		  = 0;
		states.VolumeColor.green	  = 200;
		states.VolumeColor.blue		  = 0;				
		states.SelectedCDColor.red	  = 255;
		states.SelectedCDColor.green  = 0;
		states.SelectedCDColor.blue	  = 0;				
		states.UnselectedCDColor.red  = 0;
		states.UnselectedCDColor.green= 100;
		states.UnselectedCDColor.blue = 0;				
					
	}	

	//If the last saved window position is off the screen due to a screen resize put app window
	//on the far right edge or bottom of screen if needed.
	BRect screen;
	BScreen *theScreen = new BScreen(B_MAIN_SCREEN_ID);
	screen = theScreen->Frame();

	if(states.TopLeft.x > screen.right)
		states.TopLeft.x = screen.right-192;
	if(states.TopLeft.y > screen.bottom)
		states.TopLeft.y = screen.bottom-108;
	theScreen->~BScreen();		
	
	MoveTo(states.TopLeft);

	if(!LoadBitmap(states.SkinPath,&ButtonsBitmap))
		be_app->PostMessage(B_QUIT_REQUESTED);

	aView = new AppView(Bounds());
	aView->SetViewColor(216,216,216);
	AddChild(aView);
	
	handler = aView->Window();

	handler->Show();

	SetPulseRate(1000000);		// 1 second pulse rate

	setWindow = NULL;
	

}

void AppWindow::MessageReceived(BMessage *message)
{
	switch ( message->what )
	{
	
	    case SETWIN_QUIT:
			 setWindow = NULL;
       	 	 break;			 
			 
		case NAG_BUTTON_DOWN:
			 nag_done = true;
			 break;
			 
   		default:
			 BWindow::MessageReceived(message);
			 break;
	
	}
	
}

bool AppWindow::QuitRequested()
{		
	if(setWindow != NULL)
		setWindow->Hide();
	Hide();			//Makes it look like app is closing faster :)
	
	if(states.SaveScreenPosition)
	{
		//saved to disk
		BRect			top_left;
		//Sending the top left hand point of the current window position
		top_left = Frame();
		states.TopLeft.x = top_left.left;
		states.TopLeft.y = top_left.top;	
	}
	
	SaveConfigurationFile("config.file");

	be_app->PostMessage(B_QUIT_REQUESTED);
	
	delete cd;//cd->~D_CD();	//Close up the CD stuff
	
	return(true);
}

////////////////////////////////////////////////////////////////

AppView::AppView(BRect r)
			: BView(r, "DiscJockey_VIEW", B_FOLLOW_ALL, B_WILL_DRAW | B_PULSE_NEEDED)
{	

	//cancels the option for multiple cd used at once
	states.DontStopCDWhenSwitching = 0;
	states.ShuffleAcross	 	  = 0;

	cd = new D_Cd(currentCD=states.DefaultDevice);
	states.NumDevices = cd->GetNumDevices();
	cd->ResetVolume(states.ResetVolume);
	TrackTimeType=states.DefaultTimeMode;
	
//	if(ReadKeyFile())
//		Registered =true;
//	else
//		Registered =false;
	
	if(states.ShuffleDefault)
		ShuffleCD = true;
	if(states.LoopDefault)
		LoopCD = true;
		
//	if(Registered==false)
//	{
//		nag_time_up = 120 + (rand() % 60) + (rand() % 60) + (rand() % 60);
//		nag_count = nag_time_up;		//so msg will be shown at start up
//	}


}

////////////////////////////////////////////////////////////////

void AppView::MouseDown(BPoint point)
{

	//Take care of SkipBKWD button	
	if(point.x > SkipBKWDDest.left && point.x < SkipBKWDDest.right &&
       point.y > SkipBKWDDest.top && point.y < SkipBKWDDest.bottom)		   
	{
		DrawBitmapAsync(ButtonsBitmap,SkipBKWDDownSrc,SkipBKWDDest);
		if(ShuffleCD) 	//Randomize track
	    	Skipped = true;			//Turn on track change flag	    	 	
	    else
	    	cd->SkipOneBackward();
	PlayPressed = true;	
	}
		
//  //Take care of ScanBKWD button	
//	if(point.x > SkipBKWDDest.left && point.x < SkipBKWDDest.right &&
//     point.y > SkipBKWDDest.top && point.y < SkipBKWDDest.bottom)		   
//  {
//		if(StartScaningBKWD == false)
//		{
//			StartScaningBKWD = true;		 
//			cd->StartScaningBackward();
//		}
//		else
//		{			 	
//			StartScaningBKWD = false;			 				 	
//			cd->StopScaning();	
//		}			
//	}		
//						
	//Take care of Play button		
	if(point.x > PlayDest.left && point.x < PlayDest.right &&
       point.y > PlayDest.top && point.y < PlayDest.bottom)		   
	{	
		DrawBitmapAsync(ButtonsBitmap,PlayDownSrc,PlayDest);										
		cd->PlayOrPause();
		PlayPressed = true;			
	}
		
	//Take care of Pause button		
	if(point.x > PauseDest.left && point.x < PauseDest.right &&
       point.y > PauseDest.top && point.y < PauseDest.bottom)		   
	{	
		DrawBitmapAsync(ButtonsBitmap,PauseDownSrc,PauseDest);			
		cd->Pause();			
	}

	//Take care of Stop button		
	if(point.x > StopDest.left && point.x < StopDest.right &&
       point.y > StopDest.top && point.y < StopDest.bottom)		   
	{
		DrawBitmapAsync(ButtonsBitmap,StopDownSrc,StopDest);		
		cd->Stop();			
	}

	//Take care of SkipFWD button		
	if(point.x > SkipFWDDest.left && point.x < SkipFWDDest.right &&
       point.y > SkipFWDDest.top && point.y < SkipFWDDest.bottom)		   
	{
		DrawBitmapAsync(ButtonsBitmap,SkipFWDDownSrc,SkipFWDDest);			
	    if(ShuffleCD) 	//Randomize track
			Skipped = true;			//Turn on track change flag				
	    else
		   	cd->SkipOneForward();	
	PlayPressed = true;	
	}

//	//Take care of ScanFWD button		
//	if(point.x > SkipFWDDest.left && point.x < SkipFWDDest.right &&
//     point.y > SkipFWDDest.top && point.y < SkipFWDDest.bottom)		   
//	{
//		if(StartScaningFWD == false)
//		{
//			StartScaningFWD = true;			 
//		 	cd->StartScaningForward();
//		}
//		else
//		{
//		 	StartScaningFWD = false;		
//		 	cd->StopScaning();			 	
//		}
//
//	}		
//						
	//Take care of Eject button		
	if(point.x > EjectDest.left && point.x < EjectDest.right &&
       point.y > EjectDest.top && point.y < EjectDest.bottom)		   
	{	
		DrawBitmapAsync(ButtonsBitmap,EjectDownSrc,EjectDest);
		cd->Stop();
		cd->Eject();			
	}
		
	//Take care of Shuffle Button
	if(point.x > ShuffleDest.left && point.x < ShuffleDest.right &&
       point.y > ShuffleDest.top && point.y < ShuffleDest.bottom)		   		
	{
		if(ShuffleCD == false)
		 	ShuffleCD = true;
		else
		 	ShuffleCD = false;
	}
		
	//Take care of Repeat Button
	if(point.x > RepeatDest.left && point.x < RepeatDest.right &&
       point.y > RepeatDest.top && point.y < RepeatDest.bottom)		   		
	{
		if(LoopCD == false)
		 	LoopCD = true;
		else
			LoopCD = false;					
	}
		
	//Take care of VolDown button		
	if(point.x > VolDownDest.left && point.x < VolDownDest.right &&
       point.y > VolDownDest.top && point.y < VolDownDest.bottom)		   
	{	
		DrawBitmapAsync(ButtonsBitmap,VolDown_DownSrc,VolDownDest);					
		cd->VolumeDown();		
	}
	
	//Take care of VolUp button		
	if(point.x > VolUpDest.left && point.x < VolUpDest.right &&
       point.y > VolUpDest.top && point.y < VolUpDest.bottom)		   
	{	
		DrawBitmapAsync(ButtonsBitmap,VolUp_DownSrc,VolUpDest);		
		cd->VolumeUp();				
	}

	//Take care of CD1 button
	if(point.x > CD1Dest.left && point.x < CD1Dest.right &&
       point.y > CD1Dest.top && point.y < CD1Dest.bottom)		   		
	{
		if(states.NumDevices > 0)
		{
			if(states.DontStopCDWhenSwitching==false)
				cd->Stop();
			cd->Init(CDROM_1);
			currentCD = 1;
		}
	}					
									
	//Take care of CD2 button
	if(point.x > CD2Dest.left && point.x < CD2Dest.right &&
       point.y > CD2Dest.top && point.y < CD2Dest.bottom)		   		
	{
		if(states.NumDevices > 1)
		{
			if(states.DontStopCDWhenSwitching==false)
				cd->Stop();		
			cd->Init(CDROM_2);
			currentCD = 2;
		}
	}   
		
	//Take care of CD3 button
	if(point.x > CD3Dest.left && point.x < CD3Dest.right &&
       point.y > CD3Dest.top && point.y < CD3Dest.bottom)		   		
	{
		if(states.NumDevices > 2)
		{
			if(states.DontStopCDWhenSwitching==false)
				cd->Stop();	
			cd->Init(CDROM_3);
			currentCD = 3;
		}
	}
		   
	//Take care of CD4 button
	if(point.x > CD4Dest.left && point.x < CD4Dest.right &&
       point.y > CD4Dest.top && point.y < CD4Dest.bottom)		   		
	{
		if(states.NumDevices > 3)
		{
			if(states.DontStopCDWhenSwitching==false)
				cd->Stop();
			cd->Init(CDROM_4);
			currentCD = 4;
		}
	}

	//Take care of type of track time display
	if(point.x > TimeDisplayDest.left && point.x < TimeDisplayDest.right &&
	   point.y > TimeDisplayDest.top && point.y < TimeDisplayDest.bottom)
	{
		if(TrackTimeType > 2)
			TrackTimeType=0;	
		else
			TrackTimeType++;
	}

	//Take care of About button
	if(point.x > AboutDest.left && point.x < AboutDest.right &&
       point.y > AboutDest.top && point.y < AboutDest.bottom)		   		                      
	{

	/*	if(Registered)
		{
			char custom_about[256];
			strcpy(custom_about,"DiscJockey, Copyright \xC2\xA9 1999 Hailstone Software\n");
			strcat(custom_about,"All rights reserved.\n\n");
			strcat(custom_about,"http://www.hailstonesoftware.com\n\n");
			strcat(custom_about,"Registered to: ");
			strcat(custom_about,states.RegisteredTo);
			(new BAlert(STR_ABOUT_TITLE,custom_about, STR_ABOUT_BUTTON))->Go();		
		}
		else
	*/
			(new BAlert(STR_ABOUT_TITLE,STR_ABOUT_DESC, STR_ABOUT_BUTTON))->Go();		
		
	}

	//Take care of Settings Window
	if(point.x > SettingsDest.left && point.x < SettingsDest.right &&
	   point.y > SettingsDest.top && point.y < SettingsDest.bottom)
	{
		if(setWindow != NULL)
			setWindow->Minimize(false);
 		else
 		setWindow = new SetWindow();		
	
	}

}

////////////////////////////////////////////////////////////////

void AppView::Pulse()		//going at 1 update a second
{
		
	track = cd->GetCurrentTrack();

	if( ((lastTrack != track) || (Skipped == true)) && PlayPressed==true)	//Only update track number if needed
	{	
		switch(LoopCD)
		{
			case 1:		if(ShuffleCD && (lastTrack == cd->GetNumTracks()))
						{
							int rn,tn;
							if(states.ShuffleAcross)	//shuffle across discs :)
							{
								rn = rand() % states.NumDevices +1;
								switch(rn)
								{
									case 1:	if(currentCD != 1)
											{
												cd->Stop();
												cd->Init(CDROM_1);
											}
											currentCD = 1;
											tn = cd->GetNumTracks();
											break;
									case 2: if(currentCD != 2)
											{
												cd->Stop();
												cd->Init(CDROM_2);
											}
											currentCD = 2;												
											tn = cd->GetNumTracks();								
											break;
									case 3: if(currentCD != 3)
											{
												cd->Stop();
												cd->Init(CDROM_3);
											}
											currentCD = 3;												
											tn = cd->GetNumTracks();								
											break;
									case 4: if(currentCD != 4)
											{
												cd->Stop();
												cd->Init(CDROM_4);
											}
											currentCD = 4;												
											tn = cd->GetNumTracks();								
											break;
									default: if(currentCD !=1)
											 {
												cd->Stop(); 
												cd->Init(CDROM_1);
											 }
											 currentCD = 1;																		 											
											 tn = cd->GetNumTracks();								
											 break;
								}
								
								rn = rand() % tn +1;							
								while(rn == lastTrack)	//make sure we do not keep playing the same song
									rn = rand() % tn +1;									
							}
							else
							{
								//int rn,tn;
								tn = cd->GetNumTracks();
								rn = rand() % tn +1;
								while(rn == lastTrack)	//make sure we do not keep playing the same song
									rn = rand() % tn +1;
								
							}
							cd->SelectTrack(rn);					
							lastTrack = rn;							
	
							break;		
						}
						//I think GetNumTracks causes an error with enhanced CDs
						else if(lastTrack == cd->GetNumTracks())
							 {	
							 	cd->SelectTrack(1);						//start over
							 	lastTrack = 1;			//temp fix?
				//error here with enhanced cd's	 //lastTrack = cd->GetCurrentTrack();
								break;
							 }							
					//fall through if not last track
						
			case 0:		if(ShuffleCD)
						{	
							int rn,tn;
							if(states.ShuffleAcross)	//shuffle across discs :)
							{
								rn = rand() % states.NumDevices +1;
								switch(rn)
								{
									case 1:	if(currentCD != 1)
											{
												cd->Stop();
												cd->Init(CDROM_1);
											}
											currentCD = 1;
											tn = cd->GetNumTracks();
											break;
									case 2: if(currentCD != 2)
											{
												cd->Stop();
												cd->Init(CDROM_2);
											}
											currentCD = 2;												
											tn = cd->GetNumTracks();								
											break;
									case 3: if(currentCD != 3)
											{
												cd->Stop();
												cd->Init(CDROM_3);
											}
											currentCD = 3;												
											tn = cd->GetNumTracks();								
											break;
									case 4: if(currentCD != 4)
											{
												cd->Stop();
												cd->Init(CDROM_4);
											}
											currentCD = 4;												
											tn = cd->GetNumTracks();								
											break;
									default: if(currentCD !=1)
											 {
												cd->Stop(); 
												cd->Init(CDROM_1);
											 }
											 currentCD = 1;																		 											
											 tn = cd->GetNumTracks();								
											 break;
								}
								
								rn = rand() % tn +1;							
								while(rn == lastTrack)	//make sure we do not keep playing the same song
									rn = rand() % tn +1;									
							}
							else
							{
								//int rn,tn;
								tn = cd->GetNumTracks();
								rn = rand() % tn +1;
								while(rn == lastTrack)	//make sure we do not keep playing the same song
									rn = rand() % tn +1;
								
							}
							cd->SelectTrack(rn);					
							lastTrack = rn;							

						}
						else
							lastTrack = track;
						break;
						
		}//end LoopCD switch

		Skipped = false;		//Turn off track change flag	
	
	}//end if track change

	switch(TrackTimeType)
	{
		case 0:			//amount played per track			
				minute   = cd->CurrentTrackMin();
				second   = cd->CurrentTrackSec();
				break;
		case 1:			//total amount played
				minute   = cd->CurrentTotalMin();
				second   = cd->CurrentTotalSec();			
				break;
		case 2:			//amount left per track
				minute   = cd->CurrentTrackMinLeft();
				second   = cd->CurrentTrackSecLeft();
				break;
		case 3:			//amount left total
				minute   = cd->CurrentTotalMinLeft();
				second   = cd->CurrentTotalSecLeft();
				break;
		default:		//Some how got a glitch go back to default
				TrackTimeType =0;
				break;				
	
	}

	if(second < 0)	//fixes wierd negative numbers sometimes returned
	{
		if(minute > 0)			//make sure min does not go whaco at the same time
			minute = minute--;
		else
			minute = 0;		
		second = 60 + second;
	}
	
	Draw(Bounds());

	if(Registered==false && nag_done==true)	//nag if not registered every 2 - 5 minutes
		if(nag_count == nag_time_up)
		{
			nag_done = false;
			(new BAlert("Nag message",NAG_MSG, STR_ABOUT_BUTTON))->Go(new BInvoker(new BMessage(NAG_BUTTON_DOWN),handler,NULL));
			nag_time_up = 120 + (rand() % 60) + (rand() % 60) + (rand() % 60);
			nag_count = 0;
		}
		else
			nag_count++;

}//end AppView Pulse

////////////////////////////////////////////////////////////////

void AppView::Draw(BRect updateRect)	//Redraw if covered up or something
{

	int index=0;

	//Draw display
	DrawBitmapAsync(ButtonsBitmap,DisplaySrc,TrackDisplayDest);
	DrawBitmapAsync(ButtonsBitmap,TimeDisplaySrc,TimeDisplayDest);
	int tens,ones;
	tens = minute / 10;
	ones = minute % 10;
	
	//Draw minus sign if displaying time left on track or total left
	if(TrackTimeType == 2 || TrackTimeType == 3)
		DrawBitmapAsync(ButtonsBitmap,Minus_Rect,BRect(TimePosDest.left-4,TimePosDest.top,TimePosDest.right-4,TimePosDest.bottom));
		
	//tens position of Min
	DrawBitmapAsync(ButtonsBitmap,NumberRect[tens],TimePosDest);
	//ones position of Min
	DrawBitmapAsync(ButtonsBitmap,NumberRect[ones],BRect(TimePosDest.left+4,TimePosDest.top,TimePosDest.right+4,TimePosDest.bottom));		
	tens = second / 10;
	ones = second % 10;
	//tens position of Sec
	DrawBitmapAsync(ButtonsBitmap,NumberRect[tens],BRect(TimePosDest.left+12,TimePosDest.top,TimePosDest.right+12,TimePosDest.bottom));
	//ones position of Sec
	DrawBitmapAsync(ButtonsBitmap,NumberRect[ones],BRect(TimePosDest.left+16,TimePosDest.top,TimePosDest.right+16,TimePosDest.bottom));		
	tens = lastTrack / 10;
	ones = lastTrack % 10;
	//Draw Track num
	DrawBitmapAsync(ButtonsBitmap,T_Rect,TrackPosDest);
	DrawBitmapAsync(ButtonsBitmap,R_Rect,BRect(TrackPosDest.left+4,TrackPosDest.top,TrackPosDest.right+4,TrackPosDest.bottom));
	DrawBitmapAsync(ButtonsBitmap,A_Rect,BRect(TrackPosDest.left+8,TrackPosDest.top,TrackPosDest.right+8,TrackPosDest.bottom));
	DrawBitmapAsync(ButtonsBitmap,C_Rect,BRect(TrackPosDest.left+12,TrackPosDest.top,TrackPosDest.right+12,TrackPosDest.bottom));
	DrawBitmapAsync(ButtonsBitmap,K_Rect,BRect(TrackPosDest.left+16,TrackPosDest.top,TrackPosDest.right+16,TrackPosDest.bottom));
	DrawBitmapAsync(ButtonsBitmap,NumberRect[tens],BRect(TrackPosDest.left+24,TrackPosDest.top,TrackPosDest.right+24,TrackPosDest.bottom));		
	DrawBitmapAsync(ButtonsBitmap,NumberRect[ones],BRect(TrackPosDest.left+28,TrackPosDest.top,TrackPosDest.right+28,TrackPosDest.bottom));			

	//Draw state indicators
	state = cd->GetCurrentState();
	switch(state)
	{
		case Paused:	DrawBitmapAsync(ButtonsBitmap,PauseIndicatorSrc,IndicatorPosDest);
						break;
		case Playing:	DrawBitmapAsync(ButtonsBitmap,PlayIndicatorSrc,IndicatorPosDest);
						break;
		case Stopped:	DrawBitmapAsync(ButtonsBitmap,StopIndicatorSrc,IndicatorPosDest);
						break;
	}

	//Draw CD choice block of buttons
	DrawBitmapAsync(ButtonsBitmap,CDChoiceSrc,CDPosDest);
	SetFontSize(8);
	SetHighColor(states.UnselectedCDColor);		//unselected color
	
	switch(states.NumDevices)			//Draw only devices present
	{
		default:				//If more than 4 only show 4
		case 4:	DrawChar('4',BPoint(116,22));
		case 3:	DrawChar('3',BPoint(102,22));
		case 2:	DrawChar('2',BPoint(116,10));		
		case 1:	DrawChar('1',BPoint(102,10));
				break;
	}

	SetHighColor(states.SelectedCDColor);		//selected color
	switch(currentCD)
	{
		case 1:		if(states.NumDevices > 0)
					DrawChar('1',BPoint(102,10));
					break;
		case 2:		if(states.NumDevices > 1)
					DrawChar('2',BPoint(116,10));		
					break;
		case 3:		if(states.NumDevices > 2)
					DrawChar('3',BPoint(102,22));				
					break;
		case 4:		if(states.NumDevices > 3)
					DrawChar('4',BPoint(116,22));
					break;
	}

	//Draw About
	DrawBitmapAsync(ButtonsBitmap,AboutSrc,About_SetDest);

	//DrawVolume Level
	DrawBitmapAsync(ButtonsBitmap,DisplaySrc,VolumeDisplayDest);
	float vol=0;
	float max_vol=0;
	float min_vol=0;
	
	max_vol = cd->GetMaxVolume();
	min_vol = cd->GetMinVolume();
	vol = cd->GetCurrentVolume();		
	DrawBitmapAsync(ButtonsBitmap,V_Rect,VolPosDest);
	DrawBitmapAsync(ButtonsBitmap,O_Rect,BRect(VolPosDest.left+4,VolPosDest.top,VolPosDest.right+4,VolPosDest.bottom));
	DrawBitmapAsync(ButtonsBitmap,L_Rect,BRect(VolPosDest.left+8,VolPosDest.top,VolPosDest.right+8,VolPosDest.bottom));				
	DrawBitmapAsync(ButtonsBitmap,U_Rect,BRect(VolPosDest.left+12,VolPosDest.top,VolPosDest.right+12,VolPosDest.bottom));				
	DrawBitmapAsync(ButtonsBitmap,M_Rect,BRect(VolPosDest.left+16,VolPosDest.top,VolPosDest.right+16,VolPosDest.bottom));								
	DrawBitmapAsync(ButtonsBitmap,E_Rect,BRect(VolPosDest.left+20,VolPosDest.top,VolPosDest.right+20,VolPosDest.bottom));				

	SetHighColor(states.VolumeColor);
	int total;
	int below_amount;
	float percentage;
	
		
	if(min_vol < 0)			//same as calling abs(min_vol)
		min_vol = 0 - min_vol;
	
	total = min_vol + max_vol;
	percentage = (min_vol + vol) / total;	//amount of bar to shade

	BRect	rect;		
	rect.left   = 31;
	rect.top    = 16;
	rect.bottom = 20;
	rect.right = (percentage * 59) + 31 ; //59 = end of volume bar background - 31, 31 = end of volume label and start of volume indicator bar	
	FillRect(rect);


	//Draw Buttons	
	DrawBitmapAsync(ButtonsBitmap,SkipBKWDUpSrc,SkipBKWDDest);
	DrawBitmapAsync(ButtonsBitmap,PlayUpSrc,PlayDest);
	DrawBitmapAsync(ButtonsBitmap,PauseUpSrc,PauseDest);
	DrawBitmapAsync(ButtonsBitmap,StopUpSrc,StopDest);
	DrawBitmapAsync(ButtonsBitmap,SkipFWDUpSrc,SkipFWDDest);		
	DrawBitmapAsync(ButtonsBitmap,EjectUpSrc,EjectDest);	
	DrawBitmapAsync(ButtonsBitmap,VolDown_UpSrc,VolDownDest);
	DrawBitmapAsync(ButtonsBitmap,VolUp_UpSrc,VolUpDest);
	
	if(ShuffleCD)
		DrawBitmapAsync(ButtonsBitmap,ShuffleDownSrc,ShuffleDest);
	else
		DrawBitmapAsync(ButtonsBitmap,ShuffleUpSrc,ShuffleDest);	
	
	if(LoopCD)
		DrawBitmapAsync(ButtonsBitmap,RepeatDownSrc,RepeatDest);
	else
		DrawBitmapAsync(ButtonsBitmap,RepeatUpSrc,RepeatDest);			
			

Window()->Sync();

}

////////////////////////////////////////////////////////////////

int LoadBitmap(const char *file,BBitmap **pic)
{	

	BDirectory		bdir;

	if(B_OK == bdir.SetTo(file))
		if(bdir.Contains("skin.tga"))
		{
			char 	*skinpath = new char[strlen(states.SkinPath)+strlen("skin.tga")+1];
			strcpy(skinpath,states.SkinPath);
			strcat(skinpath,"skin.tga");	
			*pic = BTranslationUtils::GetBitmapFile(skinpath,NULL);
		
			delete skinpath;
			return 1;
		}	

	char 	*defaultpath = new char[strlen(AppPath)+strlen("skins/default/skin.tga")+1];
	strcpy(defaultpath,AppPath);
	strcat(defaultpath,"skins/default/");
	bdir.SetTo(defaultpath);
	if(bdir.Contains("skin.tga"))
	{
		strcpy(states.SkinPath,defaultpath);	
		strcat(defaultpath,"skin.tga");
		*pic = BTranslationUtils::GetBitmapFile(defaultpath,NULL);		
		
		delete defaultpath;		
		return 2;
	}	
	else
	{
		delete defaultpath;
		(new BAlert("Skin ERROR:",SKIN_ERROR_MSG, STR_ABOUT_BUTTON))->Go();			
		return false;
	}
			
		
	return true;
}

////////////////////////////////////////////////////////////////

bool LoadConfigurationFile(char *filename)
{

//Load general configuration file from disk

	FILE				*fp;			//File pointer used in reading config file
	char				tmpstr[25];		//holding string for labels within the config file
	char 				*configpath = new char[strlen(AppPath)+strlen(filename)+1];
	
	strcpy(configpath,AppPath);
	strcat(configpath,filename);

	//open the disk file
	if(!(fp=fopen(configpath,"rb")))
	{	
		delete configpath;
		return false;	
	}

	fscanf(fp,"%s\n",tmpstr);
	fscanf(fp,"%s%f %f\n",tmpstr,&states.TopLeft.x,&states.TopLeft.y);
//	fscanf(fp,"%s%d\n",tmpstr,&states.DontStopCDWhenSwitching);
//	fscanf(fp,"%s%d\n",tmpstr,&states.ShuffleAcross);
	fscanf(fp,"%s%d\n",tmpstr,&states.SaveScreenPosition);
	fscanf(fp,"%s%d\n",tmpstr,&states.ResetVolume);
	fscanf(fp,"%s%d\n",tmpstr,&states.ShuffleDefault);
	fscanf(fp,"%s%d\n",tmpstr,&states.LoopDefault);	
	fscanf(fp,"%s%d\n",tmpstr,&states.DefaultDevice);
	fscanf(fp,"%s%d\n",tmpstr,&states.DefaultTimeMode);
	
	bool 				trip_wire	= false;
	char 				temp_char;
	int 				counter=0;
	int 				loop=0;
	fscanf(fp,"%s ",tmpstr);	
	while (trip_wire == false)
	{
		fscanf(fp,"%c",&temp_char);
		if(temp_char == '"')
		{
			counter++;
			if(counter == 2)
				trip_wire =true;
		}
		else
		{
			states.SkinPath[loop] = temp_char;
			loop++;
		}
	}
	states.SkinPath[loop++] = '\0';	
	
	fscanf(fp,"%s%d %d %d\n",tmpstr,&states.VolumeColor.red,&states.VolumeColor.green,&states.VolumeColor.blue);
	fscanf(fp,"%s%d %d %d\n",tmpstr,&states.SelectedCDColor.red,&states.SelectedCDColor.green,&states.SelectedCDColor.blue);
	fscanf(fp,"%s%d %d %d\n",tmpstr,&states.UnselectedCDColor.red,&states.UnselectedCDColor.green,&states.UnselectedCDColor.blue);
	
	//close the file, re-alocate ram and return success
	fclose(fp);

	delete configpath;
	
	
	return true;

}//end LoadConfigurationFile

//////////////////////////////////////////////////////////////////////////

bool SaveConfigurationFile(char *filename)
{

//Save general configuration file to disk


	FILE				*fp;			//File pointer used in reading config file
	char 				*configpath = new char[strlen(AppPath)+strlen(filename)+1];
	
	strcpy(configpath,AppPath);
	strcat(configpath,filename);

	//open the disk file
	if(!(fp=fopen(configpath,"wb")))
	{	
		(new BAlert("File save ERROR","ERROR SAVING config.file", STR_ABOUT_BUTTON))->Go();
		delete configpath;
		return false;
	}
	fprintf(fp,"%s\n","//DO_NOT_EDIT_BY_HAND!!");
	fprintf(fp,"%s %.0f %.0f\n","[WinPos]",states.TopLeft.x,states.TopLeft.y);
//	fprintf(fp,"%s %d\n","[DontStopCDWhenSwitching]",states.DontStopCDWhenSwitching);
//	fprintf(fp,"%s %d\n","[ShuffleAcross]",states.ShuffleAcross);
	fprintf(fp,"%s %d\n","[SaveScreenPos]",states.SaveScreenPosition);
	fprintf(fp,"%s %d\n","[ResetVolume]",states.ResetVolume);
	fprintf(fp,"%s %d\n","[ShuffleDefault]",states.ShuffleDefault);
	fprintf(fp,"%s %d\n","[LoopDefault]",states.LoopDefault);
	fprintf(fp,"%s %d\n","[DefaultDevice]",states.DefaultDevice);
	fprintf(fp,"%s %d\n","[DefaultTimeMode]",states.DefaultTimeMode);
	fprintf(fp,"%s %c%s%c\n","[SkinPath]",'"',states.SkinPath,'"');
	fprintf(fp,"%s %d %d %d\n","[VolumeColor]",states.VolumeColor.red,states.VolumeColor.green,states.VolumeColor.blue);
	fprintf(fp,"%s %d %d %d\n","[SelectedColor]",states.SelectedCDColor.red,states.SelectedCDColor.green,states.SelectedCDColor.blue);
	fprintf(fp,"%s %d %d %d\n","[UnselectedColor]",states.UnselectedCDColor.red,states.UnselectedCDColor.green,states.UnselectedCDColor.blue);
	
	//close the file, re-alocate ram and return success
	fclose(fp);
	delete configpath;	
	
	return true;

}//end SaveConfigurationFile

////////////////////////////////////////////////////////////////

/*
bool ReadKeyFile()
{

	char				tmpstr[25];		
	FILE				*fp;			//File pointer used in reading key file
	BPath	path;
	find_directory(B_USER_SETTINGS_DIRECTORY,&path,false,NULL);
	path.Append("DiscJockey_Key");

	//open the disk file
	if(!(fp=fopen(path.Path(),"rb")))
	{	
		return false;
	}

	fread(states.RegisteredTo,sizeof(states.RegisteredTo[0]),sizeof(states.RegisteredTo)/sizeof(states.RegisteredTo[0]),fp);
	//fscanf(fp,"%s\n",states.RegisteredTo);
	
	fclose(fp);
	return true;

}
*/

////////////////////////////////////////////////////////////////

int main(void)
{

	App		myApp;
	myApp.Run();
}
